﻿local recruitment = LibStub("AceAddon-3.0"):NewAddon("Recruitment")
local L = LibStub("AceLocale-3.0"):GetLocale("Recruitment", true)
local rcr = recruitment
rcr.frame = CreateFrame("Frame")

function rcr:msg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

local defaults = {
  profile = {
    enabled = true,
		autojoin = true,
		timebetweenads = 5,
		addrandomnumber = true,
		autorandomnumber = true,
		randomnumberrange = 5,
		disablewhenafk = true,
		disablewhendnd = true,
		manualguildname = nil,
		rawads = {},
		ads = {},
		randomnumber = 0,
		tutorialstep = 1,
		tutorialshown = false,
		guildname = nil,
		isinguild = nil,
		isincity = false,
  }
}

local timepassed = 0
local updateinterval = 3
local charname = nil
local guildname = nil
local realmname = nil
local profile = nil
local iamsender = 0
local requestid = 0
local requests = {}

function rcr:iscity()
	local zones = {
		L["CityDarnassus"],
		L["CityIronforge"],
		L["CityOrgrimmar"],
		L["CitySilvermoonCity"],
		L["CityStormwindCity"],
		L["CityTheExodar"],
		L["CityThunderBluff"],
		L["CityUndercity"]
	}
	local zonename = GetZoneText()
	self.db.profile.isincity = false
	for i, v in ipairs(zones) do
		if zonename == v then
			self.db.profile.isincity = true
		end
	end
end

function rcr:checkprofile()
end

function rcr:getguildname()
	local guildinfo = GetGuildInfo("player")
	if IsInGuild() and guildinfo then
		-- Character loaded for the first time
		-- Is in a guild but doesn't have a guild-specific profile assigned yet
		if self.db:GetCurrentProfile() == "Default" then
			profile = GUILD .. " - " .. guildinfo .. " - " .. realmname
			self.db:SetProfile(profile)
			self.db.profile.guildname = guildinfo
			guildname = guildinfo
		-- Character has a guild-specific profile assigned
		-- However, he's currently not in that guild. Must've changed guilds.
		-- Copy the former profile into a new one, remove the former
		elseif self.db:GetCurrentProfile() ~= "Default" and guildname ~= guildinfo then
			if not guildname then
				profile = GUILD .. " - " .. guildinfo .. " - " .. realmname
				self.db:SetProfile(profile)
				self.db.profile.guildname = guildinfo
				guildname = guildinfo
				return
			end
			local tempprofile = profile
			profile = GUILD .. " - " .. guildinfo .. " - " .. realmname
			if tempprofile == profile then return end
			self.db:SetProfile(profile)
			self.db:CopyProfile(tempprofile, true)
			self.db:DeleteProfile(tempprofile, true)
			profile = self.db:GetCurrentProfile()
		end
		self.db.profile.isinguild = true
	else
		if self.db:GetCurrentProfile() == "Default" then
			profile = GUILD .. " - " .. charname .. " - " .. realmname
			self.db:SetProfile(profile)
			self.db.profile.guildname = charname
		end
	end
end

function rcr:tutorialstep()
	local tutorial = L["AdsStep1"]
	local example = string.gsub(L["AdsStepExample1"], "%(%(g%)%)", L["Heroes"])
	if self.db.profile.tutorialstep == 2 then
		tutorial = L["AdsStep2"]
		example = L["AdsStepExample1"]
	elseif self.db.profile.tutorialstep == 3 then
		tutorial = L["AdsStep3"]
		example = string.gsub(L["AdsStepExample2"], "|", "||")
	elseif self.db.profile.tutorialstep == 4 then
		tutorial = L["AdsStep4"]
		example = L["AdsStepExample2"] .. "\n\n" .. L["AdsStepExample3"]
		example = string.gsub(example, "|", "||")
	elseif self.db.profile.tutorialstep == 5 then
		tutorial = L["AdsStep5"]
		example = "2 " .. L["AdsStepExample2"]
		example = string.gsub(example, "|", "||")
	elseif self.db.profile.tutorialstep == 6 then
		tutorial = L["AdsStep6"]
		example = "0 " .. L["AdsStepExample2"]
		example = string.gsub(example, "|", "||")
	elseif self.db.profile.tutorialstep == 7 then
		tutorial = L["AdsStep7"]
		example = string.gsub(L["AdsStepExample2"], "|", "||")
	end
	return tutorial, example
end

function rcr:send()
	local randomnumber = math.random(self.db.profile.randomnumber)
	for i, v in ipairs(self.db.profile.ads) do
		if randomnumber >= self.db.profile.ads[i][3] and randomnumber <= self.db.profile.ads[i][4] then
			self:msg(self.db.profile.ads[i][2])
			self:msg(randomnumber)
			return
		end
	end
end

function rcr:sendaddon(request)
	if request and iamsender == 0 then
		local uptime = math.ceil(GetTime())
		requestid = uptime .. math.random(100)
		local priority = math.random(100)
		table.insert(requests, {id=requestid, time=uptime, priority=priority})
		SendAddonMessage("Recruitment", "request(" .. requestid .. ", " .. priority, "guild")
end

function rcr:getaddon(prefix, msg, channel, sender)
	self:msg("prefix = " .. (prefix or "nil"))
	self:msg("msg = " .. (msg or "nil"))
	self:msg("channel = " .. (channel or "nil"))
	self:msg("sender = " .. (sender or "nil"))
end

function rcr.events(self, event, ...)
	rcr:msg("event: " .. event)
	if event == "CHAT_MSG_CHANNEL_NOTICE" then
		local channeljoined, _, _, _, _, _, _, _, channelname = ...
		if channeljoined == "YOU_JOINED" and string.match(channelname, L["GuildRecruitment"]) then
		end
	elseif event == "CHAT_MSG_ADDON" and ... == "Recruitment" then
		rcr:getaddon(...)
	elseif event == "PLAYER_GUILD_UPDATE" or event == "GUILD_ROSTER_UPDATE" then
		rcr:getguildname()
	elseif event == "ZONE_CHANGED_NEW_AREA" then
		if rcr:iscity() then
		end
	end
end

function rcr.updates(self, elapsed)
	timepassed = timepassed + elapsed
	if timepassed > updateinterval then
		timepassed = 0
		-- rcr:send()
	end
end

rcr.frame:SetScript("OnEvent", rcr.events)
rcr.frame:SetScript("OnUpdate", rcr.updates)

function rcr:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("recruitmentdb", defaults, true)
	local options = {
		name = "Recruitment",
		handler = recruitment,
		type = "group",
		childGroups = "tab",
		args = {
			header = {
				order = 10,
				type = "description",
				name = L["Notes"],
				image = "Interface\\Icons\\Achievement_GuildPerk_EverybodysFriend",
				imageWidth = 32,
				imageHeight = 32,
			},
			enabled = {
				order = 20,
				type = "toggle",
				tristate = true,
				name = L["Enabled"],
				desc = L["EnabledDesc"],
				get = function() return self.db.profile.enabled end,
				set = function()
					if self.db.profile.enabled then
						self.db.profile.enabled = false
						self:Disable()
					else
						self.db.profile.enabled = true
						self:Enable()
					end
				end,
			},
			generalsettings = {
				order = 30,
				type = "group",
				name = L["GeneralSettings"],
				desc = L["GeneralSettingsHeadline"],
				args = {
					autojoin = {
						order = 10,
						type = "toggle",
						name = L["AutoJoin"],
						desc = L["AutoJoinDesc"],
						width = "full",
						get = function() return self.db.profile.autojoin end,
						set = function(info, value) self.db.profile.autojoin = value end,
					},
					interval = {
						order = 20,
						type = "range",
						name = L["Interval"],
						desc = L["IntervalDesc1"] .. "\n\n" .. L["IntervalDesc2"],
						width = "full",
						min = 1,
						max = 60,
						step = 1,
						get = function() return self.db.profile.timebetweenads end,
						set = function(info, value) self.db.profile.timebetweenads = value end,
					},
					addrandomnumber = {
						order = 30,
						type = "toggle",
						name = L["RandomNumber"],
						desc = L["RandomNumberDesc1"] .. "\n\n" .. L["RandomNumberDesc2"],
						width = "full",
						get = function() return self.db.profile.addrandomnumber end,
						set = function(info, value) self.db.profile.addrandomnumber = value end,
					},
					autorandomnumber = {
						order = 40,
						type = "toggle",
						name = L["AutoRandomNumber"],
						desc = L["AutoRandomNumberDesc1"] .. "\n\n" .. L["AutoRandomNumberDesc2"] .. "\n\n" .. L["AutoRandomNumberDesc3"],
						width = "full",
						disabled = function() return not self.db.profile.addrandomnumber end,
						get = function() return self.db.profile.autorandomnumber end,
						set = function(info, value) self.db.profile.autorandomnumber = value end,
					},
					manualrandomnumber = {
						order = 50,
						type = "range",
						name = L["ManualRandomNumber"],
						desc = L["ManualRandomNumberDesc1"] .. "\n\n" .. L["ManualRandomNumberDesc2"],
						width = "full",
						disabled = function() return not self.db.profile.addrandomnumber or self.db.profile.autorandomnumber end,
						min = 1,
						max = 60,
						step = 1,
						get = function() return self.db.profile.randomnumberrange end,
						set = function(info, value) self.db.profile.randomnumberrange = value end,
					},
					disablewhenafk = {
						order = 60,
						type = "toggle",
						name = L["DisableWhenAFK"],
						desc = L["DisableWhenAFKDesc1"] .. "\n\n" .. L["DisableWhenAFKDesc2"],
						width = "full",
						get = function() return self.db.profile.disablewhenafk end,
						set = function(info, value) self.db.profile.disablewhenafk = value end,
					},
					disablewhendnd = {
						order = 70,
						type = "toggle",
						name = L["DisableWhenDND"],
						desc = L["DisableWhenDNDDesc1"] .. "\n\n" .. L["DisableWhenDNDDesc2"],
						width = "full",
						get = function() return self.db.profile.disablewhendnd end,
						set = function(info, value) self.db.profile.disablewhendnd = value end,
					},
				},
			},
			ads = {
				order = 40,
				type = "group",
				name = L["Ads"],
				desc = L["AdsHeadline"],
				args = {
					manualguildname = {
						order = 10,
						type = "input",
						name = L["ManualGuildName"],
						desc = L["ManualGuildNameDesc1"] .. "\n\n" .. L["ManualGuildNameDesc2"],
						width = "full",
						hidden = function() return self.db.profile.isinguild end,
						get = function() return self.db.profile.manualguildname end,
						set = function(info, value)
							local currentprofiles = self.db:GetProfiles()
							local currentprofile = self.db:GetCurrentProfile()
							self.db:SetProfile(GUILD .. " - " .. value .. " - " .. realmname)
							self.db:CopyProfile(currentprofile, true)
							self.db.profile.guildname = value
							self.db.profile.manualguildname = value
							for i, v in ipairs(currentprofiles) do
								if v == GUILD .. " - " .. charname .. " - " .. realmname then
									self.db:DeleteProfile(v, true)
								end
							end
						end,
					},
					textbox = {
						order = 20,
						type = "input",
						name = L["Ads"],
						desc = L["AdsDesc"],
						width = "full",
						multiline = true,
						get = function() return self.db.profile.rawads end,
						set = function(info, value)
							local helper = value .. "\n"
							local counter = 0
							self.db.profile.ads = {}
							self.db.profile.randomnumber = 0
							for v in string.gmatch(helper, "(%C-)%c") do
								counter = counter + 1
								table.insert(self.db.profile.ads, counter, {
									tonumber(string.match(v, "^(%d*)%s?")),
									string.match(v, "^%d*%s?(.*)"),
									tonumber(string.match(v, "^(%d*)%s?")) == 0 and 0 or (self.db.profile.randomnumber + 1),
									tonumber(string.match(v, "^(%d*)%s?")) == 0 and 0 or (self.db.profile.randomnumber + (tonumber(string.match(v, "^(%d*)%s?")) or 1))	
								})
								self.db.profile.randomnumber = self.db.profile.randomnumber + (self.db.profile.ads[counter][1] or 1)
							end
							self.db.profile.rawads = value
						end,
					},
					adstutorialheadline = {
						order = 30,
						type = "description",
						name = "\n" .. L["AdsStepByStep"] .. ":\n",
					},
					adstutorialshowhide = {
						order = 40,
						type = "execute",
						name = function()
							if self.db.profile.tutorialshown then
								return L["AdsTutorialHide"]
							end
							return L["AdsTutorialShow"]
						end,
						desc = function()
							if self.db.profile.tutorialshown then
								return L["AdsTutorialHideDesc"]
							end
							return L["AdsTutorialShowDesc"]
						end,
						func = function() self.db.profile.tutorialshown = not self.db.profile.tutorialshown end,
					},
					adstutorialback = {
						order = 41,
						type = "execute",
						name = L["AdsTutorialBack"],
						desc = L["AdsTutorialBackDesc"],
						width = "half",
						func = function() self.db.profile.tutorialstep = self.db.profile.tutorialstep - 1 end,
						disabled = function() if self.db.profile.tutorialstep == 1 then return true end end,
						hidden = function() return not self.db.profile.tutorialshown end,
					},
					adstutorialnext = {
						order = 42,
						type = "execute",
						name = L["AdsTutorialNext"],
						desc = L["AdsTutorialNextDesc"],
						width = "half",
						func = function() self.db.profile.tutorialstep = self.db.profile.tutorialstep + 1 end,
						disabled = function() if self.db.profile.tutorialstep == 7 then return true end end,
						hidden = function() return not self.db.profile.tutorialshown end,
					},
					adstutorialstep = {
						order = 50,
						type = "description",
						name = function() return "\n" .. rcr:tutorialstep() end,
						hidden = function() return not self.db.profile.tutorialshown end,
					},
					adstutorialexamplepre = {
						order = 55,
						type = "header",
						name = "",
						hidden = function() return not self.db.profile.tutorialshown end,
					},
					adstutorialexample = {
						order = 60,
						type = "description",
						name = function() local _, example = rcr:tutorialstep() return example end,
						hidden = function() return not self.db.profile.tutorialshown end,
					},
				},
			},
		},
	}
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Recruitment", options, nil)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Recruitment")
	self.db.RegisterCallback(self, "OnProfileChanged", "checkprofile")
	self.db.RegisterCallback(self, "OnProfileCopied", "checkprofile")
	self.db.RegisterCallback(self, "OnProfileReset", "checkprofile")
end

function rcr:OnEnable()
	if not charname then charname = UnitName("player") end
	if not realmname then realmname = GetRealmName() end
	self:getguildname()
	self:iscity()
	self:sendaddon()
	if not self.db.profile.enabled then self:Disable() return end
	rcr.frame:RegisterEvent("CHAT_MSG_ADDON")
	rcr.frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	rcr.frame:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	rcr.frame:RegisterEvent("PLAYER_GUILD_UPDATE")
	rcr.frame:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:msg("an")
end

function rcr:OnDisable()
	rcr.frame:UnregisterAllEvents()
	self:msg("aus")
end